@@ -2,6 +2,7 @@
 # :copyright: Copyright (c) 2015 ftrack
 
 import collections
+import functools
 
 import ftrack_api.entity.base
 import ftrack_api.exception
@@ -356,7 +357,15 @@ class Location(ftrack_api.entity.base.Entity):
                 source.get_resource_identifier(component), 'rb'
             )
             target_data = self.accessor.open(resource_identifier, 'wb')
-            target_data.write(source_data.read())
+
+            # Read/write data in chunks to avoid reading all into memory at the
+            # same time.
+            chunked_read = functools.partial(
+                source_data.read, ftrack_api.symbol.CHUNK_SIZE
+            )
+            for chunk in iter(chunked_read, ''):
+                target_data.write(chunk)
+
             target_data.close()
             source_data.close()
 
@@ -701,3 +710,23 @@ class OriginLocationMixin(MemoryLocationMixin, UnmanagedLocationMixin):
             ))
 
         return context
+
+
+class ServerLocationMixin(object):
+    '''Location representing ftrack server.
+
+    Adds convenience methods to location, specific to ftrack server.
+    '''
+    def get_thumbnail_url(self, component, size=None):
+        '''Return thumbnail url for *component*.
+
+        Optionally, specify *size* to constrain the downscaled image to size
+        x size pixels.
+
+        Raise :exc:`~ftrack_api.exception.AccessorFilesystemPathError` if
+        URL could not be determined from *resource_identifier* or
+        :exc:`~ftrack_api.exception.AccessorUnsupportedOperationError` if
+        retrieving URL is not supported by the location's accessor.
+        '''
+        resource_identifier = self.get_resource_identifier(component)
+        return self.accessor.get_thumbnail_url(resource_identifier, size)
